/*
******************************************************************************

   LibOVL: extension stubs

   Copyright (C) 2000 Christoph Egger   [Christoph_Egger@t-online.de]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <ggi/internal/ovl.h>
#include <ggi/internal/ovl_debug.h>
#include <ggi/internal/galloc_ops.h>


ggiOvl_t ggiOvlTie(ggi_visual_t vis,
		   ggiGA_resource_handle handle_motor,
                   ggiGA_resource_handle handle_carb,
                   ggiGA_resource_handle handle_tank)

{
	int rc = GALLOC_OK;
	ggiOvl_t ovl;
	
	if (handle_motor == NULL
	    || handle_tank == NULL
	    || handle_carb == NULL)
	{
		/* invalid argument */
		goto err0;
	}	/* if */

	if (ggiGAIsFailed(handle_motor)
	   || ggiGAIsFailed(handle_tank)
	   || ggiGAIsFailed(handle_carb))
	{
		/* libGAlloc failed! */
		goto err0;
	}	/* if */


	ovl = calloc(1, sizeof(*ovl));
	if (!ovl) {
		/* not enough memory */
		goto err0;
	}	/* if */

	/* Copy resource at handle to ovl->res_handle_tank.
	 * Note: ggiGAAppend() is black-ops.  We should perhaps
	 * have something in grey-ops if not the external API 
	 * to copy a single resource, minus the ->next.
	 */
	ovl->res_handle_motor = NULL;
	ggiGAAppend(handle_motor, &(ovl->res_handle_motor));
	if (ovl->res_handle_motor == NULL) goto err1;

	ovl->res_handle_tank = NULL;
	ggiGAAppend(handle_tank, &(ovl->res_handle_tank));
	if (ovl->res_handle_tank == NULL) goto err2;

	ovl->res_handle_carb = NULL;
	ggiGAAppend(handle_carb, &(ovl->res_handle_carb));
	if (ovl->res_handle_carb == NULL) goto err3;

	ovl->vis = ggiGAGetVisual(ovl->res_handle_tank);
	LIB_ASSERT(ovl->vis != NULL);
	ovl->res_type = ggiGAGetType(ovl->res_handle_tank);
	ovl->props = ggiGAGetProperties(ovl->res_handle_tank);


	/* We position the ovl at (0,0), which is the top-left-corner
	 * of the screen.
	 */
	ovl->x = ovl->cur_x = 0;
	ovl->y = ovl->cur_y = 0;

	LIB_ASSERT(ovl->x != GGI_AUTO);
	LIB_ASSERT(ovl->y != GGI_AUTO);

	ovl->w = ovl->cur_w = ovl->props->size.area.x;
	ovl->h = ovl->cur_h = ovl->props->size.area.y;

	ovl->hidecount = 1;	/* Come up hidden! */
	ovl->disablecount = 0;	/* Come up enabled (active) */
	ovl->zoomed = 0;	/* not zoomed */
	ovl->slot = 0;		/* default value */

	ovl->cb = ggiGAGetCoordBase(ovl->res_handle_tank);

	rc = LIBGGI_OVLEXT(ovl->vis)->tie(vis, ovl);

	LIB_ASSERT(rc == GALLOC_OK);
	if (rc != GALLOC_OK) {
		/* something went wrong */
		goto err2;
	}	/* if */

	LIB_ASSERT(rc == GALLOC_OK);
	return ovl;

err3:
	ggiGAEmptyList(&(ovl->res_handle_tank));
err2:
	ggiGAEmptyList(&(ovl->res_handle_motor));
err1:
	free(ovl);
err0:
	return NULL;
}	/* ggiOvlTie */


int ggiOvlUntie(ggiOvl_t *ovl)
{
	int rc = GALLOC_OK;

	if (ovl == NULL) return GGI_EARGINVAL;
	if (ovl[0] == NULL) return GGI_EARGINVAL;

	while (!ggiOvlIsHidden(ovl[0])) {
		rc = ggiOvlHide(ovl[0]);
		LIB_ASSERT(rc == GALLOC_OK);
		if (rc != GALLOC_OK) {
			/* something went wrong here */
			return rc;
		}	/* if */
	}	/* while */
	

	if (ovl[0]->alpha != NULL) {
		free(ovl[0]->alpha);
		ovl[0]->alpha = NULL;
		ovl[0]->alpha_size = 0;
	}	/* if */


	/* release the resource */
	ovl[0]->props = NULL;
	ggiGAEmptyList(&(ovl[0]->res_handle_motor));
	ggiGAEmptyList(&(ovl[0]->res_handle_tank));
	ggiGAEmptyList(&(ovl[0]->res_handle_carb));

	free(ovl[0]);
	ovl = NULL;

	LIB_ASSERT(rc == GALLOC_OK);
	return rc;
}	/* ggiOvlUntie */


int ggiOvlSetImage(ggiOvl_t ovl, ggi_visual_t vis,
		   long x, long y, unsigned char *alpha)

{
	return LIBGGI_OVLEXT(ovl->vis)->setimage(ovl, vis, x, y, alpha);
}	/* ggiOvlSetImage */



int ggiOvlSetPos(ggiOvl_t ovl, long x, long y)
{
	return LIBGGI_OVLEXT(ovl->vis)->setpos(ovl, x, y);
}	/* ggiOvlSetPos */


int ggiOvlMove(ggiOvl_t ovl, long x, long y)
{
	return LIBGGI_OVLEXT(ovl->vis)->move(ovl, x, y);
}	/* ggiOvlMove */




int ggiOvlSetSize(ggiOvl_t ovl, long w, long h)
{
	return LIBGGI_OVLEXT(ovl->vis)->setsize(ovl, w, h);
}	/* ggiOvlSetSize */


int ggiOvlSetPriority(ggiOvl_t ovl, uint32 slot, int swap)
{
	return LIBGGI_OVLEXT(ovl->vis)->setpriority(ovl, slot, swap);
}	/* ggiOvlSetPriority */



int ggiOvlShow(ggiOvl_t ovl)
{
	return LIBGGI_OVLEXT(ovl->vis)->show(ovl);
}	/* ggiOvlShow */


int ggiOvlHide(ggiOvl_t ovl)
{
	return LIBGGI_OVLEXT(ovl->vis)->hide(ovl);
}	/* ggiOvlHide */


int ggiOvlIsHidden(ggiOvl_t ovl)
{
	return (ovl->hidecount > 0) ? 1 : 0;
}	/* ggiOvlIsHidden */




int ggiOvlEnable(ggiOvl_t ovl)
{
	return LIBGGI_OVLEXT(ovl->vis)->enable(ovl);
}	/* ggiOvlEnable */


int ggiOvlDisable(ggiOvl_t ovl)
{
	return LIBGGI_OVLEXT(ovl->vis)->disable(ovl);
}	/* ggiOvlDisable */


int ggiOvlIsEnabled(ggiOvl_t ovl)
{
	return (ovl->disablecount > 0) ? 0 : 1;
}	/* ggiOvlIsEnabled */



int ggiOvlSetZoom(ggiOvl_t ovl, long x, long y, long w, long h, int keep_ratio)
{
	return LIBGGI_OVLEXT(ovl->vis)->setzoom(ovl, x,y, w,h, keep_ratio);
}	/* ggiOvlSetZoom */

